@@ -1,10 +1,7 @@
 class BaseRedmineError(Exception):
     """Base exception class for Redmine exceptions"""
-    def __init__(self, message):
-        self.message = message
-
-    def __str__(self):
-        return self.message
+    def __init__(self, *args, **kwargs):
+        super(BaseRedmineError, self).__init__(*args, **kwargs)
 
 
 class ResourceError(BaseRedmineError):
@@ -13,6 +10,18 @@ class ResourceError(BaseRedmineError):
         super(ResourceError, self).__init__('Unsupported redmine resource')
 
 
+class NoFileError(BaseRedmineError):
+    """File doesn't exist exception"""
+    def __init__(self):
+        super(NoFileError, self).__init__("Can't upload the file that doesn't exist")
+
+
+class ResourceNotFoundError(BaseRedmineError):
+    """Resource not found"""
+    def __init__(self):
+        super(ResourceNotFoundError, self).__init__("Requested resource doesn't exist")
+
+
 class AuthError(BaseRedmineError):
     """Invalid authentication details"""
     def __init__(self):
@@ -25,6 +34,18 @@ class ImpersonateError(BaseRedmineError):
         super(ImpersonateError, self).__init__("Impersonate login provided doesn't exist or isn't active")
 
 
+class ServerError(BaseRedmineError):
+    """Redmine internal error"""
+    def __init__(self):
+        super(ServerError, self).__init__('Redmine returned internal error, perhaps you are doing something wrong')
+
+
+class ValidationError(BaseRedmineError):
+    """Redmine validation error"""
+    def __init__(self, error):
+        super(ValidationError, self).__init__(error)
+
+
 class ResourceSetIndexError(BaseRedmineError):
     """Index doesn't exist in the ResourceSet"""
     def __init__(self):
@@ -34,7 +55,7 @@ class ResourceSetIndexError(BaseRedmineError):
 class ResourceSetFilterParamError(BaseRedmineError):
     """Resource set filter method expects to receive either a list or tuple"""
     def __init__(self):
-        super(ResourceSetFilterParamError, self).__init__("Method expects to receive either a list or tuple of ids")
+        super(ResourceSetFilterParamError, self).__init__('Method expects to receive either a list or tuple of ids')
 
 
 class ResourceBadMethodError(BaseRedmineError):
@@ -52,7 +73,13 @@ class ResourceFilterError(BaseRedmineError):
 class ResourceNoFiltersProvidedError(BaseRedmineError):
     """No filter(s) provided"""
     def __init__(self):
-        super(ResourceNoFiltersProvidedError, self).__init__("Resource needs some filters to be filtered on")
+        super(ResourceNoFiltersProvidedError, self).__init__('Resource needs some filters to be filtered on')
+
+
+class ResourceNoFieldsProvidedError(BaseRedmineError):
+    """No field(s) provided"""
+    def __init__(self):
+        super(ResourceNoFieldsProvidedError, self).__init__('Resource needs some fields to be created')
 
 
 class ResourceAttrError(BaseRedmineError):
@@ -61,7 +88,13 @@ class ResourceAttrError(BaseRedmineError):
         super(ResourceAttrError, self).__init__("Resource doesn't have the requested attribute")
 
 
-class ResourceVersionMismatchError(BaseRedmineError):
+class VersionMismatchError(BaseRedmineError):
+    """Feature isn't supported on specified Redmine version"""
+    def __init__(self, feature):
+        super(VersionMismatchError, self).__init__("{0} isn't supported on specified Redmine version".format(feature))
+
+
+class ResourceVersionMismatchError(VersionMismatchError):
     """Resource isn't supported on specified Redmine version"""
     def __init__(self):
-        super(ResourceVersionMismatchError, self).__init__("Resource isn't supported on specified Redmine version")
+        super(ResourceVersionMismatchError, self).__init__('Resource')
