@@ -1,11 +1,15 @@
 from distutils.version import LooseVersion
 from redmine.resultsets import ResourceSet
+from redmine.utilities import MemorizeFormatter
 from redmine.exceptions import (
     ResourceError,
     ResourceBadMethodError,
     ResourceFilterError,
     ResourceNoFiltersProvidedError,
-    ResourceVersionMismatchError
+    ResourceNoFieldsProvidedError,
+    ResourceVersionMismatchError,
+    ResourceNotFoundError,
+    ValidationError
 )
 
 
@@ -24,14 +28,14 @@ class ResourceManager(object):
         except AttributeError:
             raise ResourceError()
 
-        if redmine.ver is not None and LooseVersion(str(redmine.ver)) < LooseVersion(resource_class.version):
+        if redmine.ver is not None and LooseVersion(str(redmine.ver)) < LooseVersion(resource_class.redmine_version):
             raise ResourceVersionMismatchError()
 
         self.redmine = redmine
         self.resource_class = resource_class
 
-    def request(self, method, **params):
-        """A proxy for Redmine object request which also does some extra work for resource retrieval"""
+    def retrieve(self, **params):
+        """A proxy for Redmine object request which does some extra work for resource retrieval"""
         self.params.update(**params)
 
         # Redmine allows us to only return 100 resources per request, so if
@@ -45,7 +49,7 @@ class ResourceManager(object):
                 self.params['limit'] = 100
 
                 while True:
-                    response = self.redmine.request(method, self.url, **self.params)
+                    response = self.redmine.request('get', self.url, params=self.params)
 
                     if response is None:
                         break
@@ -58,7 +62,7 @@ class ResourceManager(object):
             # we want to get > 100 resources
             else:
                 while self.params['limit'] > 0:
-                    response = self.redmine.request(method, self.url, **self.params)
+                    response = self.redmine.request('get', self.url, params=self.params)
 
                     if response is None:
                         break
@@ -69,7 +73,7 @@ class ResourceManager(object):
 
             return results
 
-        return self.redmine.request(method, self.url, **self.params)[self.container]
+        return self.redmine.request('get', self.url, params=self.params)[self.container]
 
     def to_resource(self, resource):
         """Converts a single resource dict from Redmine result set to resource object"""
@@ -87,11 +91,7 @@ class ResourceManager(object):
         self.url = '{0}{1}'.format(self.redmine.url, self.resource_class.query_one.format(resource_id, **params))
         self.params = params
         self.container = self.resource_class.container_one
-
-        try:
-            return self.resource_class(self, self.request('get'))
-        except TypeError:
-            return None
+        return self.resource_class(self, self.retrieve())
 
     def all(self, **params):
         """Returns a ResourceSet object with all Resource objects"""
@@ -123,3 +123,44 @@ class ResourceManager(object):
 
         self.params = filters
         return ResourceSet(self)
+
+    def create(self, **fields):
+        """Creates a new resource in Redmine database and returns resource object on success"""
+        if self.resource_class.query_create is None or self.resource_class.container_create is None:
+            raise ResourceBadMethodError()
+
+        if not fields:
+            raise ResourceNoFieldsProvidedError
+
+        if 'uploads' in fields:
+            for index, upload in enumerate(fields['uploads']):
+                fields['uploads'][index]['token'] = self.redmine.upload(upload.get('path', ''))
+
+        formatter = MemorizeFormatter()
+
+        try:
+            url = '{0}{1}'.format(self.redmine.url, formatter.format(self.resource_class.query_create, **fields))
+        except KeyError as exception:
+            raise ValidationError('{0} field is required'.format(exception))
+
+        self.container = self.resource_class.container_one
+        data = {self.resource_class.container_create: formatter.unused_kwargs}
+
+        # Almost all resources are created via POST method, but some
+        # resources are created via PUT, so we should check for this
+        try:
+            response = self.redmine.request('post', url, data=data)
+        except ResourceNotFoundError:
+            response = self.redmine.request('put', url, data=data)
+
+        if self.container in response:
+            resource = self.resource_class(self, response[self.container])
+        else:
+            raise ValidationError('Resource already exists')  # fix for repeated PUT requests
+
+        self.params = formatter.used_kwargs
+        self.url = '{0}{1}'.format(
+            self.redmine.url,
+            self.resource_class.query_one.format(resource.internal_id, **fields)
+        )
+        return resource
