@@ -1,5 +1,5 @@
-import sys
 from datetime import datetime
+from redmine.utilities import to_string
 from redmine.managers import ResourceManager
 from redmine.exceptions import ResourceAttrError
 
@@ -30,6 +30,7 @@ _RESOURCE_MAP = {
     'status': 'IssueStatus',
     'user': 'User',
     'issue': 'Issue',
+    'priority': 'Enumeration',
     'activity': 'Enumeration',
     'category': 'IssueCategory',
 }
@@ -50,13 +51,15 @@ _RESOURCE_RELATIONS_MAP = {
 
 class _Resource(object):
     """Implementation of Redmine resource"""
-    version = None
+    redmine_version = None
     container_all = None
     container_one = None
     container_filter = None
+    container_create = None
     query_all = None
     query_one = None
     query_filter = None
+    query_create = None
 
     _relations = {}
     __length_hint__ = None  # fixes Python 2.6 list() call on resource object
@@ -113,6 +116,11 @@ class _Resource(object):
         """Returns full url to the resource for humans"""
         return self.manager.url.replace('.json', '')
 
+    @property
+    def internal_id(self):
+        """Returns identifier of the resource for usage in internals of the library"""
+        return self.id
+
     def __dir__(self):
         """We need to show only real Redmine resource attributes on dir() call"""
         return list(self.attributes.keys())
@@ -121,22 +129,32 @@ class _Resource(object):
         """Provides a way to iterate through resource attributes and its values"""
         return iter(self.attributes.items())
 
+    def __int__(self):
+        """Integer representation of the Redmine resource object"""
+        return self.id
+
+    def __str__(self):
+        """Informal representation of the Redmine resource object"""
+        return to_string(self.name)
+
     def __repr__(self):
-        """Official representation of the Redmine resource"""
+        """Official representation of the Redmine resource object"""
         return '<{0}.{1} #{2} "{3}">'.format(
             self.__class__.__module__,
             self.__class__.__name__,
             self.id,
-            self.name.encode('utf-8') if sys.version_info[0] < 3 else self.name
+            to_string(self.name)
         )
 
 
 class Project(_Resource):
-    version = '1.0'
+    redmine_version = '1.0'
     container_all = 'projects'
     container_one = 'project'
+    container_create = 'project'
     query_all = '/projects.json'
     query_one = '/projects/{0}.json'
+    query_create = '/projects.json'
 
     _relations = {
         'wiki_pages': None,
@@ -149,26 +167,34 @@ class Project(_Resource):
 
 
 class Issue(_Resource):
-    version = '1.0'
+    redmine_version = '1.0'
     container_all = 'issues'
     container_one = 'issue'
     container_filter = 'issues'
+    container_create = 'issue'
     query_all = '/issues.json'
     query_one = '/issues/{0}.json'
     query_filter = '/issues.json'
+    query_create = '/projects/{project_id}/issues.json'
 
     _relations = {
         'relations': None,
         'time_entries': None,
     }
 
+    def __str__(self):
+        try:
+            return to_string(self.subject)
+        except ResourceAttrError:
+            return str(self.id)
+
     def __repr__(self):
         try:
             return '<{0}.{1} #{2} "{3}">'.format(
                 self.__class__.__module__,
                 self.__class__.__name__,
                 self.id,
-                self.subject.encode('utf-8') if sys.version_info[0] < 3 else self.subject
+                to_string(self.subject)
             )
         except ResourceAttrError:
             return '<{0}.{1} #{2}>'.format(
@@ -179,13 +205,18 @@ class Issue(_Resource):
 
 
 class TimeEntry(_Resource):
-    version = '1.1'
+    redmine_version = '1.1'
     container_all = 'time_entries'
     container_one = 'time_entry'
     container_filter = 'time_entries'
+    container_create = 'time_entry'
     query_all = '/time_entries.json'
     query_one = '/time_entries/{0}.json'
     query_filter = '/issues/{issue_id}/time_entries.json'
+    query_create = '/time_entries.json'
+
+    def __str__(self):
+        return str(self.id)
 
     def __repr__(self):
         return '<{0}.{1} #{2}>'.format(
@@ -196,27 +227,33 @@ class TimeEntry(_Resource):
 
 
 class Enumeration(_Resource):
-    version = '2.2'
+    redmine_version = '2.2'
     container_filter = '{resource}'
     query_filter = '/enumerations/{resource}.json'
 
 
 class Attachment(_Resource):
-    version = '1.3'
+    redmine_version = '1.3'
     container_one = 'attachment'
     query_one = '/attachments/{0}.json'
 
+    def __str__(self):
+        return to_string(self.filename)
+
     def __repr__(self):
         return '<{0}.{1} #{2} "{3}">'.format(
             self.__class__.__module__,
             self.__class__.__name__,
             self.id,
-            self.filename.encode('utf-8') if sys.version_info[0] < 3 else self.filename
+            to_string(self.filename)
         )
 
 
 class IssueJournal(_Resource):
-    version = '1.0'
+    redmine_version = '1.0'
+
+    def __str__(self):
+        return str(self.id)
 
     def __repr__(self):
         return '<{0}.{1} #{2}>'.format(
@@ -227,29 +264,46 @@ class IssueJournal(_Resource):
 
 
 class WikiPage(_Resource):
-    version = '2.2'
+    redmine_version = '2.2'
     container_filter = 'wiki_pages'
     container_one = 'wiki_page'
+    container_create = 'wiki_page'
     query_filter = '/projects/{project_id}/wiki/index.json'
     query_one = '/projects/{project_id}/wiki/{0}.json'
+    query_create = '/projects/{project_id}/wiki/{title}.json'
 
     def refresh(self):
-        return self.manager.get(self.title, **{'project_id': self.manager.params['project_id']})
+        return self.manager.get(self.title, project_id=self.manager.params['project_id'])
+
+    @property
+    def internal_id(self):
+        return self.title
+
+    def __int__(self):
+        return self.version
+
+    def __str__(self):
+        return to_string(self.title)
 
     def __repr__(self):
         return '<{0}.{1} "{2}">'.format(
             self.__class__.__module__,
             self.__class__.__name__,
-            self.title.encode('utf-8') if sys.version_info[0] < 3 else self.title
+            to_string(self.title)
         )
 
 
 class ProjectMembership(_Resource):
-    version = '1.4'
+    redmine_version = '1.4'
     container_filter = 'memberships'
     container_one = 'membership'
+    container_create = 'membership'
     query_filter = '/projects/{project_id}/memberships.json'
     query_one = '/memberships/{0}.json'
+    query_create = '/projects/{project_id}/memberships.json'
+
+    def __str__(self):
+        return str(self.id)
 
     def __repr__(self):
         return '<{0}.{1} #{2}>'.format(
@@ -260,19 +314,26 @@ class ProjectMembership(_Resource):
 
 
 class IssueCategory(_Resource):
-    version = '1.3'
+    redmine_version = '1.3'
     container_filter = 'issue_categories'
     container_one = 'issue_category'
+    container_create = 'issue_category'
     query_filter = '/projects/{project_id}/issue_categories.json'
     query_one = '/issue_categories/{0}.json'
+    query_create = '/projects/{project_id}/issue_categories.json'
 
 
 class IssueRelation(_Resource):
-    version = '1.3'
+    redmine_version = '1.3'
     container_filter = 'relations'
     container_one = 'relation'
+    container_create = 'relation'
     query_filter = '/issues/{issue_id}/relations.json'
     query_one = '/relations/{0}.json'
+    query_create = '/issues/{issue_id}/relations.json'
+
+    def __str__(self):
+        return str(self.id)
 
     def __repr__(self):
         return '<{0}.{1} #{2}>'.format(
@@ -283,21 +344,31 @@ class IssueRelation(_Resource):
 
 
 class Version(_Resource):
-    version = '1.3'
+    redmine_version = '1.3'
     container_filter = 'versions'
     container_one = 'version'
+    container_create = 'version'
     query_filter = '/projects/{project_id}/versions.json'
     query_one = '/versions/{0}.json'
+    query_create = '/projects/{project_id}/versions.json'
 
 
 class User(_Resource):
-    version = '1.1'
+    redmine_version = '1.1'
     container_all = 'users'
     container_one = 'user'
     container_filter = 'users'
+    container_create = 'user'
     query_all = '/users.json'
     query_one = '/users/{0}.json'
     query_filter = '/users.json'
+    query_create = '/users.json'
+
+    def __str__(self):
+        try:
+            return super(User, self).__str__()
+        except ResourceAttrError:
+            return '{0} {1}'.format(to_string(self.firstname), to_string(self.lastname))
 
     def __repr__(self):
         try:
@@ -307,21 +378,23 @@ class User(_Resource):
                 self.__class__.__module__,
                 self.__class__.__name__,
                 self.id,
-                self.firstname.encode('utf-8') if sys.version_info[0] < 3 else self.firstname,
-                self.lastname.encode('utf-8') if sys.version_info[0] < 3 else self.lastname
+                to_string(self.firstname),
+                to_string(self.lastname)
             )
 
 
 class Group(_Resource):
-    version = '2.1'
+    redmine_version = '2.1'
     container_all = 'groups'
     container_one = 'group'
+    container_create = 'group'
     query_all = '/groups.json'
     query_one = '/groups/{0}.json'
+    query_create = '/groups.json'
 
 
 class Role(_Resource):
-    version = '1.4'
+    redmine_version = '1.4'
     container_all = 'roles'
     container_one = 'role'
     query_all = '/roles.json'
@@ -329,7 +402,7 @@ class Role(_Resource):
 
 
 class News(_Resource):
-    version = '1.1'
+    redmine_version = '1.1'
     container_all = 'news'
     container_filter = 'news'
     query_all = '/news.json'
@@ -337,25 +410,25 @@ class News(_Resource):
 
 
 class IssueStatus(_Resource):
-    version = '1.3'
+    redmine_version = '1.3'
     container_all = 'issue_statuses'
     query_all = '/issue_statuses.json'
 
 
 class Tracker(_Resource):
-    version = '1.3'
+    redmine_version = '1.3'
     container_all = 'trackers'
     query_all = '/trackers.json'
 
 
 class Query(_Resource):
-    version = '1.3'
+    redmine_version = '1.3'
     container_all = 'queries'
     query_all = '/queries.json'
 
 
 class CustomField(_Resource):
-    version = '2.4'
+    redmine_version = '2.4'
     container_all = 'custom_fields'
     query_all = '/custom_fields.json'
 
